@@ -99,13 +99,14 @@ config = {
 			"final_score": { "type": "float"},
 			"sid": { "type": "keyword"},
 			"id": { "type": "keyword"},
-			"rid": { "type": "integer"},
+			"rid": { "type": "keyword"},
 			"did": { "type": "integer"},
 			"ibeg": { "type": "integer"},
 			"iend": { "type": "integer"},
+			"len": { "type": "integer"}, # [start, start + len) 
 			"docid": { "type": "keyword"},
-			"uid": { "type": "integer"},
-			"eid": { "type": "integer"},
+			"uid": { "type": "keyword"},
+			"eid": { "type": "keyword"},
 			"sntnum": { "type": "integer"},
 			"wordnum": { "type": "integer"},
 			"awl": { "type": "float"},
@@ -119,6 +120,8 @@ config = {
 			"low": { "type": "keyword"},
 			"pos": { "type": "keyword"},
 			"tag": { "type": "keyword"},
+			"ctag": { "type": "keyword"}, # chunk tag
+			"stag": { "type": "keyword"}, # snt tag
 			"term": { "type": "keyword"},
 			"gpos": { "type": "keyword"},
 			"glem": { "type": "keyword"},
