@@ -272,7 +272,7 @@ GET my_index/_search
       "script": {
         "source": "cosineSimilarity(params.query_vector, 'my_vector') + 1.0",
         "params": {
-          "query_vector": [ 1,1,2,3]
+          "query_vector": [ 1,2,3]
         }
       }
     }
